if not ElvUI then return end;
if not ElvUI[1] then return end;
if not BastasTools then return end;

local E = ElvUI[1];
local UF = E:GetModule('UnitFrames');
local LSM = LibStub("LibSharedMedia-3.0")

--WoW API / Variables
local UnitIsUnit = UnitIsUnit
local UnitReaction = UnitReaction
local UnitIsPlayer = UnitIsPlayer
local UnitClass = UnitClass
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local FACTION_BAR_COLORS = FACTION_BAR_COLORS

local function Color(r,g,b,a)
	local t = {};
	t.r = r;
	t.g = g;
	t.b = b;
	t.a = a or 1;
	return t;
end

local colors = {};
colors.dk		= Color(.77,.12,.23);
colors.dh 		= Color(.64,.19,.79);
colors.druid 	= Color(1,.49,.04);
colors.hunter 	= Color(.67,.83,.45);
colors.mage 	= Color(.41,.8,.94);
colors.monk		= Color(0,1,.59);
colors.paladin	= Color(.96,.55,.73);
colors.priest	= Color(1,1,1);
colors.rogue	= Color(1,.96,.41);
colors.shaman	= Color(0,.44,.87);
colors.warlock	= Color(.58,.51,.79);
colors.warrior	= Color(.78,.61,.43);
colors.moon		= Color(.8,.8,1);

local iconColor = {};
iconColor[1] = colors.rogue;
iconColor[2] = colors.druid;
iconColor[3] = colors.warlock;
iconColor[4] = colors.hunter;
iconColor[5] = colors.moon;
iconColor[6] = colors.mage;
iconColor[7] = colors.dk;
iconColor[8] = colors.priest;

local auraColor = {
	["Empowered Pulse Grenade"] = colors.moon,
	["Charged Blasts"] = colors.dk,
	["Necrotic Embrace"] = colors.dk
}

_ENABLE_ICON_GLOW = false;
_ENABLE_AURA_GLOW = true;
_ENABLE_REJUV_GLOW = false;
_REJUV_PANDEMIC = 20*0.3;
_FRAME_PERIODIC_UPDATE = 0.3;
_OVERLAY_ALPHA = 0.3;

function UF:UpdateCustomGlow()
	if not self.unit then return; end
	
	local unit = self.unit
	local icon = GetRaidTargetIndex(unit);
	local _flag = false;
	local _flag2 = false;
	
	if ( _ENABLE_ICON_GLOW and icon ) then
		local c = iconColor[icon];
		_flag = true;
		_flag2 = true;
		self.CustomGlow:SetBackdropBorderColor(c.r, c.g, c.b);
		self.CustomGlow.powerGlow:SetBackdropBorderColor(c.r, c.g, c.b);
	end
	if ( _ENABLE_REJUV_GLOW and unit and UnitExists(unit) and not UnitIsDead(unit) ) then
		local name,_,_,_,_,_,expires = UnitAura(unit,"Rejuvenation");
		if ( not name or expires-GetTime() <= _REJUV_PANDEMIC ) then
			local c = colors.monk;
			_flag = true;
			self.CustomGlow:SetBackdropBorderColor(c.r, c.g, c.b);
			self.CustomGlow.powerGlow:SetBackdropBorderColor(c.r, c.g, c.b);
		end		
	end
	if ( _ENABLE_AURA_GLOW and unit and UnitExists(unit) and not UnitIsDead(unit) ) then
		for aura,c in pairs(auraColor) do
			if (UnitAura(unit,aura) or UnitDebuff(unit,aura)) then
				_flag = true;
				self.CustomGlow:SetBackdropBorderColor(c.r, c.g, c.b);
				self.CustomGlow.powerGlow:SetBackdropBorderColor(c.r, c.g, c.b);
				break;
			end
		end		
	end	

	if (not _flag) then
		self.CustomGlow:Hide();
		self.CustomGlow.powerGlow:Hide();
	else
		self.CustomGlow:Show();
		if self.USE_POWERBAR_OFFSET then
			self.CustomGlow.powerGlow:Show()
		end
	end
	
	--[[
	if (not _flag2) then
		self.btoverlay:Hide();
	else
		self.btoverlay:Show();
	end
	--]]
end

local _total = 0;
function UF:UpdateCustomGlowTimer(elapsed)
	_total = _total + elapsed;
	if ( _total >= _FRAME_PERIODIC_UPDATE ) then
		_total = 0;
		UF.UpdateCustomGlow(self);
	end
end

local t = 2;
local function Configure_CustomGlow(frame)
	--if not frame.VARIABLES_SET or not frame.CustomGlow then return end
	SHADOW_SPACING = t*3; --frame.SHADOW_SPACING;
	local targetHealthGlow = frame.CustomGlow
	local targetPowerGlow = frame.CustomGlow.powerGlow
	targetHealthGlow:ClearAllPoints()
	if frame.USE_POWERBAR_OFFSET then
		if frame.ORIENTATION == "RIGHT" then
			targetHealthGlow:Point("TOPLEFT", frame.Health.backdrop, "TOPLEFT", -SHADOW_SPACING - frame.SPACING, SHADOW_SPACING + frame.SPACING + (frame.USE_CLASSBAR and (frame.USE_MINI_CLASSBAR and 0 or frame.CLASSBAR_HEIGHT) or 0))
			targetHealthGlow:Point("BOTTOMRIGHT", frame.Health.backdrop, "BOTTOMRIGHT", SHADOW_SPACING + frame.SPACING, -SHADOW_SPACING - frame.SPACING)
		else
			targetHealthGlow:Point("TOPLEFT", frame.Health.backdrop, "TOPLEFT", -SHADOW_SPACING - frame.SPACING, SHADOW_SPACING + frame.SPACING + (frame.USE_CLASSBAR and (frame.USE_MINI_CLASSBAR and 0 or frame.CLASSBAR_HEIGHT) or 0))
			targetHealthGlow:Point("BOTTOMRIGHT", frame.Health.backdrop, "BOTTOMRIGHT", SHADOW_SPACING + frame.SPACING, -SHADOW_SPACING - frame.SPACING)
		end

		targetPowerGlow:ClearAllPoints()
		targetPowerGlow:Point("TOPLEFT", frame.Power.backdrop, "TOPLEFT", -SHADOW_SPACING - frame.SPACING, SHADOW_SPACING + frame.SPACING)
		targetPowerGlow:Point("BOTTOMRIGHT", frame.Power.backdrop, "BOTTOMRIGHT", SHADOW_SPACING + frame.SPACING, -SHADOW_SPACING - frame.SPACING)
	else
		targetHealthGlow:Point("TOPLEFT", -SHADOW_SPACING, SHADOW_SPACING-(frame.USE_MINI_CLASSBAR and frame.CLASSBAR_YOFFSET or 0))

		if frame.USE_MINI_POWERBAR then
			targetHealthGlow:Point("BOTTOMLEFT", -SHADOW_SPACING, -SHADOW_SPACING + (frame.POWERBAR_HEIGHT/2))
			targetHealthGlow:Point("BOTTOMRIGHT", SHADOW_SPACING, -SHADOW_SPACING + (frame.POWERBAR_HEIGHT/2))
		else
			targetHealthGlow:Point("BOTTOMLEFT", -SHADOW_SPACING, -SHADOW_SPACING)
			targetHealthGlow:Point("BOTTOMRIGHT", SHADOW_SPACING, -SHADOW_SPACING)
		end
	end
	
	
	
end

local function CreateThickShadow(f)
	if f.shadow then return end
	borderr, borderg, borderb = 0, 0, 0
	backdropr, backdropg, backdropb = 0, 0, 0

	local shadow = CreateFrame("Frame", nil, f)
	shadow:SetOutside(f, t*3, t*3)
	shadow:SetFlattensRenderLayers(false);
	shadow:SetBackdrop( {
		edgeFile = LSM:Fetch("border", "ElvUI GlowBorder"), edgeSize = t*3,
		insets = {left = t*5, right = t*5, top = t*5, bottom = t*5},
	})
	shadow:SetBackdropColor(backdropr, backdropg, backdropb, 0)
	shadow:SetBackdropBorderColor(borderr, borderg, borderb, 0.9)
	f.shadow = shadow
	return f;
end

local function CreateOverlay(f)
	if f.btoverlay then return end;
	
	f.btoverlay = f:CreateTexture();
	f.btoverlay:SetAllPoints(f)
	f.btoverlay:SetTexture(1,1,1,0);
	--f.btoverlay:SetFrameStrata("HIGH");
	--f.btoverlay:SetFrameLevel(1);
	f.btoverlay:Hide();
end

function UF:Construct_CustomGlow(frame)
	--CreateOverlay(frame);
	
	frame = CreateThickShadow(frame)
	local x = frame.shadow
	frame.shadow = nil
	x:SetFrameStrata("HIGH")
	x:SetFrameLevel(1)
	x:Hide();

	--Secondary TargetGlow, for power frame when using power offset
	frame = CreateThickShadow(frame)
	local shadow = frame.shadow
	shadow:SetFrameStrata("HIGH")
	shadow:SetFrameLevel(1)
	x.powerGlow = shadow
	frame.shadow = nil
	
	return x
end

local function Construct_RaidFrames_CustomGlow(self)
	if ( self and self.unitframeType == "raid" and not self._initCustomGlow) then
		self._initCustomGlow = true;
		self.CustomGlow = UF:Construct_CustomGlow(self);
		Configure_CustomGlow(self);
		--tinsert(self.__elements, UF.UpdateTargetGlow)
		self:RegisterEvent("UNIT_AURA")
		self:HookScript("UNIT_AURA",UF.UpdateCustomGlow)
		self:RegisterEvent("RAID_TARGET_UPDATE")
		self:HookScript("RAID_TARGET_UPDATE",UF.UpdateCustomGlow)
	end
end

local function Construct_GlowFrames(...)
    if ElvUF then
        for _,frame in pairs(ElvUF.objects) do
			if (frame and frame.unit and not frame._initCustomGlow and
				(frame.unitframeType == "raid" or frame.unitframeType == "party" or frame.unitframeType == "raid40")
            ) then
				frame._initCustomGlow = true;
				frame.CustomGlow = UF:Construct_CustomGlow(frame);
				Configure_CustomGlow(frame);
				frame:RegisterEvent("UNIT_AURA",UF.UpdateCustomGlow)
				frame:RegisterEvent("RAID_TARGET_UPDATE",UF.UpdateCustomGlow)
				frame:SetScript("OnUpdate",UF.UpdateCustomGlowTimer);
            end
        end
    end
end

local BTElvGlowFrame = CreateFrame("Frame");
BTElvGlowFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
BTElvGlowFrame:RegisterEvent("RAID_ROSTER_UPDATE");
BTElvGlowFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
BTElvGlowFrame:SetScript("OnEvent",Construct_GlowFrames);


--[[
MENU STUFF
--]]
local BTGlowMenuFrame = CreateFrame("Frame", "BTGlowMenuFrame", UIParent);

local function AddButton(index, parent, label, callbackstr)
	local button = CreateFrame("CheckButton", "UICheckButtonBT" .. index, parent, "UICheckButtonTemplate");
    --button:ClearAllPoints();
	button:SetPoint("BOTTOMLEFT", parent, "BOTTOMLEFT", 8, 8+(index-1)*16);
	button:SetWidth(16);
	button:SetHeight(16);
	button:SetChecked(_G[callbackstr]);
	button:SetScript("OnClick",function(self)
		_G[callbackstr] = self:GetChecked();
	end);	
	_G[button:GetName().."Text"]:SetText(label);
	_G[button:GetName().."Text"]:SetTextColor(1,1,1,1);	
	button:Show();
end

BTGlowMenuFrame:SetBackdrop({
      bgFile="Interface\\DialogFrame\\UI-DialogBox-Background", 
      edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
      tile=1, tileSize=8, edgeSize=8, 
      insets={left=2, right=2, top=2, bottom=2}
})

BTGlowMenuFrame:SetWidth(128)
BTGlowMenuFrame:SetHeight(64)
BTGlowMenuFrame:SetPoint("CENTER", UIParent)
BTGlowMenuFrame:EnableMouse(true)
BTGlowMenuFrame:SetMovable(true)
BTGlowMenuFrame:RegisterForDrag("LeftButton")
BTGlowMenuFrame:SetFrameStrata("FULLSCREEN_DIALOG")
BTGlowMenuFrame:Hide();

--BTGlowMenuFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
--BTGlowMenuFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
BTGlowMenuFrame:SetScript("OnMouseDown", function(self, button)
  if button == "LeftButton" and not self.isMoving then
   self:StartMoving();
   self.isMoving = true;
  end
end)
BTGlowMenuFrame:SetScript("OnMouseUp", function(self, button)
  if button == "LeftButton" and self.isMoving then
   self:StopMovingOrSizing();
   self.isMoving = false;
  end
end)
BTGlowMenuFrame:SetScript("OnHide", function(self)
  if ( self.isMoving ) then
   self:StopMovingOrSizing();
   self.isMoving = false;
  end
end)

AddButton(1, BTGlowMenuFrame," Icons","_ENABLE_ICON_GLOW");
AddButton(2, BTGlowMenuFrame," Raid Debuffs","_ENABLE_AURA_GLOW");
AddButton(3, BTGlowMenuFrame," Missing Rejuv","_ENABLE_REJUV_GLOW");

function BastasTools:UpdateElvGlowOptionsVisibility()
	if ( self.db.global.showGlowOptionsPanel ) then
		BTGlowMenuFrame:Show();
	else 
		BTGlowMenuFrame:Hide();
	end
end

function BastasTools:ToggleElvGlowMenu()
	self.db.global.showGlowOptionsPanel = not self.db.global.showGlowOptionsPanel;
	self:UpdateElvGlowOptionsVisibility();
end

function BastasTools:InitElvGlow()
	self:UpdateElvGlowOptionsVisibility();
end












